=== modified file 'bzrlib/btree_index.py'
--- bzrlib/btree_index.py	2008-11-28 02:32:40 +0000
+++ bzrlib/btree_index.py	2009-02-18 05:40:39 +0000
@@ -801,6 +801,19 @@
             new_tips = next_tips
         return final_offsets
 
+    def external_references(self, ref_list_num):
+        if self._root_node is None:
+            self._get_root_node()
+        if ref_list_num + 1 > self.node_ref_lists:
+            raise ValueError('No ref list %d, index has %d ref lists'
+                % (ref_list_num, self.node_ref_lists))
+        keys = set()
+        refs = set()
+        for node in self.iter_all_entries():
+            keys.add(node[1])
+            refs.update(node[3][ref_list_num])
+        return refs - keys
+
     def _find_layer_first_and_end(self, offset):
         """Find the start/stop nodes for the layer corresponding to offset.
 

=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2009-02-18 03:01:40 +0000
+++ bzrlib/index.py	2009-02-18 05:40:39 +0000
@@ -435,10 +435,13 @@
             # there must be one line - the empty trailer line.
             raise errors.BadIndexData(self)
 
-    def _external_references(self, ref_list_num=None):
+    def external_references(self, ref_list_num=None):
         """Return references that are not present in this index.
         """
         self._buffer_all()
+        if ref_list_num + 1 > self.node_ref_lists:
+            raise ValueError('No ref list %d, index has %d ref lists'
+                % (ref_list_num, self.node_ref_lists))
         refs = set()
         for key, (value, ref_lists) in self._nodes.iteritems():
             ref_list = ref_lists[ref_list_num]

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2009-02-18 03:01:40 +0000
+++ bzrlib/knit.py	2009-02-18 05:40:39 +0000
@@ -2274,7 +2274,7 @@
         :param graph_index: A GraphIndex
         """
         self._missing_compression_parents.update(
-            graph_index._external_references(ref_list_num=1))
+            graph_index.external_references(ref_list_num=1))
         self._missing_compression_parents.difference_update(
             self.get_parent_map(self._missing_compression_parents))
 

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2008-11-09 18:40:17 +0000
+++ bzrlib/tests/test_btree_index.py	2009-02-18 05:40:39 +0000
@@ -869,6 +869,37 @@
             (index, ('name', 'fin2'), 'beta', ((), ))]),
             set(index.iter_entries_prefix([('name', None)])))
 
+    # XXX: external_references tests are duplicated in test__index.  We
+    # probably should have per_graph_index tests...
+    def test_external_references_no_refs(self):
+        index = self.make_index(ref_lists=0, nodes=[])
+        self.assertRaises(ValueError, index.external_references, 0)
+
+    def test_external_references_no_results(self):
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key',), 'value', ([],))])
+        self.assertEqual(set(), index.external_references(0))
+
+    def test_external_references_missing_ref(self):
+        missing_key = ('missing',)
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key',), 'value', ([missing_key],))])
+        self.assertEqual(set([missing_key]), index.external_references(0))
+
+    def test_external_references_multiple_ref_lists(self):
+        missing_key = ('missing',)
+        index = self.make_index(ref_lists=2, nodes=[
+            (('key',), 'value', ([], [missing_key]))])
+        self.assertEqual(set([]), index.external_references(0))
+        self.assertEqual(set([missing_key]), index.external_references(1))
+
+    def test_external_references_two_records(self):
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key-1',), 'value', ([('key-2',)],)),
+            (('key-2',), 'value', ([],)),
+            ])
+        self.assertEqual(set([]), index.external_references(0))
+
 
 class TestBTreeNodes(BTreeTestCase):
 

=== modified file 'bzrlib/tests/test_index.py'
--- bzrlib/tests/test_index.py	2008-12-16 14:58:29 +0000
+++ bzrlib/tests/test_index.py	2009-02-18 05:40:39 +0000
@@ -922,6 +922,39 @@
         index = self.make_index(nodes=[(('key', ), 'value', ())])
         index.validate()
 
+    # XXX: external_references tests are duplicated in test_btree_index.  We
+    # probably should have per_graph_index tests...
+    def test_external_references_no_refs(self):
+        index = self.make_index(ref_lists=0, nodes=[])
+        self.assertRaises(ValueError, index.external_references, 0)
+
+    def test_external_references_no_results(self):
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key',), 'value', ([],))])
+        self.assertEqual(set(), index.external_references(0))
+
+    def test_external_references_missing_ref(self):
+        missing_key = ('missing',)
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key',), 'value', ([missing_key],))])
+        self.assertEqual(set([missing_key]), index.external_references(0))
+
+    def test_external_references_multiple_ref_lists(self):
+        missing_key = ('missing',)
+        index = self.make_index(ref_lists=2, nodes=[
+            (('key',), 'value', ([], [missing_key]))])
+        self.assertEqual(set([]), index.external_references(0))
+        self.assertEqual(set([missing_key]), index.external_references(1))
+
+    def test_external_references_two_records(self):
+        index = self.make_index(ref_lists=1, nodes=[
+            (('key-1',), 'value', ([('key-2',)],)),
+            (('key-2',), 'value', ([],)),
+            ])
+        self.assertEqual(set([]), index.external_references(0))
+
+
+
 
 class TestCombinedGraphIndex(TestCaseWithMemoryTransport):
 

